@@ -5,8 +5,8 @@
 
     Lexers for web-related languages and markup.
 
-    :copyright: 2006-2007 by Georg Brandl, Armin Ronacher,
-                Tim Hatch <tim@timhatch.com>.
+    :copyright: 2006-2008 by Georg Brandl, Armin Ronacher,
+                Tim Hatch <tim@timhatch.com>, Stou Sandalski.
     :license: BSD, see LICENSE for more details.
 """
 
@@ -24,7 +24,7 @@ from pygments.util import get_bool_opt, get_list_opt, looks_like_xml, \
 
 
 __all__ = ['HtmlLexer', 'XmlLexer', 'JavascriptLexer', 'CssLexer',
-           'PhpLexer']
+           'PhpLexer', 'ActionScriptLexer', 'XsltLexer', 'ActionScript3Lexer']
 
 
 class JavascriptLexer(RegexLexer):
@@ -48,8 +48,8 @@ class JavascriptLexer(RegexLexer):
             (r'[~\^\*!%&<>\|+=:;,/?\\-]+', Operator),
             (r'[{}\[\]();.]+', Punctuation),
             (r'(for|in|while|do|break|return|continue|if|else|throw|try|'
-             r'catch|var|with|const|label|function|new|typeof|'
-             r'instanceof|this)\b', Keyword),
+             r'catch|new|typeof|instanceof|this)\b', Keyword),
+            (r'(var|with|const|label|function)\b', Keyword.Declaration),
             (r'(true|false|null|NaN|Infinity|undefined)\b', Keyword.Constant),
             (r'(Array|Boolean|Date|Error|Function|Math|netscape|'
              r'Number|Object|Packages|RegExp|String|sun|decodeURI|'
@@ -58,6 +58,90 @@ class JavascriptLexer(RegexLexer):
              r'window)\b', Name.Builtin),
             (r'[$a-zA-Z_][a-zA-Z0-9_]*', Name.Other),
             (r'[0-9][0-9]*\.[0-9]+([eE][0-9]+)?[fd]?', Number.Float),
+            (r'0x[0-9a-fA-F]+', Number.Hex),
+            (r'[0-9]+', Number.Integer),
+            (r'"(\\\\|\\"|[^"])*"', String.Double),
+            (r"'(\\\\|\\'|[^'])*'", String.Single),
+        ]
+    }
+
+
+class ActionScriptLexer(RegexLexer):
+    """
+    For ActionScript source code.
+
+    *New in Pygments 0.9.*
+    """
+
+    name = 'ActionScript'
+    aliases = ['as', 'actionscript']
+    filenames = ['*.as']
+    mimetypes = ['application/x-actionscript', 'text/x-actionscript',
+                 'text/actionscript']
+
+    flags = re.DOTALL
+    tokens = {
+        'root': [
+            (r'\s+', Text),
+            (r'//.*?\n', Comment),
+            (r'/\*.*?\*/', Comment),
+            (r'/(\\\\|\\/|[^/\n])*/[gim]*', String.Regex),
+            (r'[~\^\*!%&<>\|+=:;,/?\\-]+', Operator),
+            (r'[{}\[\]();.]+', Punctuation),
+            (r'(case|default|for|each|in|while|do|break|return|continue|if|else|'
+             r'throw|try|catch|var|with|new|typeof|arguments|instanceof|this|'
+             r'switch)\b', Keyword),
+            (r'(class|public|final|internal|native|override|private|protected|'
+             r'static|import|extends|implements|interface|intrinsic|return|super|'
+             r'dynamic|function|const|get|namespace|package|set)\b',
+             Keyword.Declaration),
+            (r'(true|false|null|NaN|Infinity|-Infinity|undefined|Void)\b',
+             Keyword.Constant),
+            (r'(Accessibility|AccessibilityProperties|ActionScriptVersion|'
+             r'ActivityEvent|AntiAliasType|ApplicationDomain|AsBroadcaster|Array|'
+             r'AsyncErrorEvent|AVM1Movie|BevelFilter|Bitmap|BitmapData|'
+             r'BitmapDataChannel|BitmapFilter|BitmapFilterQuality|BitmapFilterType|'
+             r'BlendMode|BlurFilter|Boolean|ByteArray|Camera|Capabilities|CapsStyle|'
+             r'Class|Color|ColorMatrixFilter|ColorTransform|ContextMenu|'
+             r'ContextMenuBuiltInItems|ContextMenuEvent|ContextMenuItem|'
+             r'ConvultionFilter|CSMSettings|DataEvent|Date|DefinitionError|'
+             r'DeleteObjectSample|Dictionary|DisplacmentMapFilter|DisplayObject|'
+             r'DisplacmentMapFilterMode|DisplayObjectContainer|DropShadowFilter|'
+             r'Endian|EOFError|Error|ErrorEvent|EvalError|Event|EventDispatcher|'
+             r'EventPhase|ExternalInterface|FileFilter|FileReference|'
+             r'FileReferenceList|FocusDirection|FocusEvent|Font|FontStyle|FontType|'
+             r'FrameLabel|FullScreenEvent|Function|GlowFilter|GradientBevelFilter|'
+             r'GradientGlowFilter|GradientType|Graphics|GridFitType|HTTPStatusEvent|'
+             r'IBitmapDrawable|ID3Info|IDataInput|IDataOutput|IDynamicPropertyOutput'
+             r'IDynamicPropertyWriter|IEventDispatcher|IExternalizable|'
+             r'IllegalOperationError|IME|IMEConversionMode|IMEEvent|int|'
+             r'InteractiveObject|InterpolationMethod|InvalidSWFError|InvokeEvent|'
+             r'IOError|IOErrorEvent|JointStyle|Key|Keyboard|KeyboardEvent|KeyLocation|'
+             r'LineScaleMode|Loader|LoaderContext|LoaderInfo|LoadVars|LocalConnection|'
+             r'Locale|Math|Matrix|MemoryError|Microphone|MorphShape|Mouse|MouseEvent|'
+             r'MovieClip|MovieClipLoader|Namespace|NetConnection|NetStatusEvent|'
+             r'NetStream|NewObjectSample|Number|Object|ObjectEncoding|PixelSnapping|'
+             r'Point|PrintJob|PrintJobOptions|PrintJobOrientation|ProgressEvent|Proxy|'
+             r'QName|RangeError|Rectangle|ReferenceError|RegExp|Responder|Sample|Scene|'
+             r'ScriptTimeoutError|Security|SecurityDomain|SecurityError|'
+             r'SecurityErrorEvent|SecurityPanel|Selection|Shape|SharedObject|'
+             r'SharedObjectFlushStatus|SimpleButton|Socket|Sound|SoundChannel|'
+             r'SoundLoaderContext|SoundMixer|SoundTransform|SpreadMethod|Sprite|'
+             r'StackFrame|StackOverflowError|Stage|StageAlign|StageDisplayState|'
+             r'StageQuality|StageScaleMode|StaticText|StatusEvent|String|StyleSheet|'
+             r'SWFVersion|SyncEvent|SyntaxError|System|TextColorType|TextField|'
+             r'TextFieldAutoSize|TextFieldType|TextFormat|TextFormatAlign|'
+             r'TextLineMetrics|TextRenderer|TextSnapshot|Timer|TimerEvent|Transform|'
+             r'TypeError|uint|URIError|URLLoader|URLLoaderDataFormat|URLRequest|'
+             r'URLRequestHeader|URLRequestMethod|URLStream|URLVariabeles|VerifyError|'
+             r'Video|XML|XMLDocument|XMLList|XMLNode|XMLNodeType|XMLSocket|XMLUI)\b',
+             Name.Builtin),
+            (r'(decodeURI|decodeURIComponent|encodeURI|escape|eval|isFinite|isNaN|'
+             r'isXMLName|clearInterval|fscommand|getTimer|getURL|getVersion|'
+             r'isFinite|parseFloat|parseInt|setInterval|trace|updateAfterEvent|'
+             r'unescape)\b',Name.Function),
+            (r'[$a-zA-Z_][a-zA-Z0-9_]*', Name.Other),
+            (r'[0-9][0-9]*\.[0-9]+([eE][0-9]+)?[fd]?', Number.Float),
             (r'0x[0-9a-f]+', Number.Hex),
             (r'[0-9]+', Number.Integer),
             (r'"(\\\\|\\"|[^"])*"', String.Double),
@@ -66,6 +150,73 @@ class JavascriptLexer(RegexLexer):
     }
 
 
+class ActionScript3Lexer(RegexLexer):
+    """
+    For ActionScript 3 source code.
+
+    *New in Pygments 0.11.*
+    """
+
+    name = 'ActionScript 3'
+    aliases = ['as3', 'actionscript3']
+    filenames = ['*.as']
+    mimetypes = ['application/x-actionscript', 'text/x-actionscript',
+                 'text/actionscript']
+
+    identifier = r'[$a-zA-Z_][a-zA-Z0-9_]*'
+
+    flags = re.DOTALL | re.MULTILINE
+    tokens = {
+        'root': [
+            (r'\s+', Text),
+            (r'(function\s+)(' + identifier + r')(\s*)(\()',
+             bygroups(Keyword.Declaration, Name.Function, Text, Operator),
+             'funcparams'),
+            (r'(var|const)(\s+)(' + identifier + r')(\s*)(:)(\s*)(' + identifier + r')',
+             bygroups(Keyword.Declaration, Text, Name, Text, Punctuation, Text,
+                      Keyword.Type)),
+            (r'(import|package)(\s+)((?:' + identifier + r'|\.)+)(\s*)',
+             bygroups(Keyword, Text, Name.Namespace, Text)),
+            (r'(new)(\s+)(' + identifier + r')(\s*)(\()',
+             bygroups(Keyword, Text, Keyword.Type, Text, Operator)),
+            (r'//.*?\n', Comment.Single),
+            (r'/\*.*?\*/', Comment.Multiline),
+            (r'/(\\\\|\\/|[^\n])*/[gisx]*', String.Regex),
+            (r'(\.)(' + identifier + r')', bygroups(Operator, Name.Attribute)),
+            (r'(case|default|for|each|in|while|do|break|return|continue|if|else|'
+             r'throw|try|catch|with|new|typeof|arguments|instanceof|this|'
+             r'switch|import|include|as|is)\b',
+             Keyword),
+            (r'(class|public|final|internal|native|override|private|protected|'
+             r'static|import|extends|implements|interface|intrinsic|return|super|'
+             r'dynamic|function|const|get|namespace|package|set)\b',
+             Keyword.Declaration),
+            (r'(true|false|null|NaN|Infinity|-Infinity|undefined|Void)\b',
+             Keyword.Constant),
+            (r'(decodeURI|decodeURIComponent|encodeURI|escape|eval|isFinite|isNaN|'
+             r'isXMLName|clearInterval|fscommand|getTimer|getURL|getVersion|'
+             r'isFinite|parseFloat|parseInt|setInterval|trace|updateAfterEvent|'
+             r'unescape)\b', Name.Function),
+            (identifier, Name),
+            (r'[0-9][0-9]*\.[0-9]+([eE][0-9]+)?[fd]?', Number.Float),
+            (r'0x[0-9a-f]+', Number.Hex),
+            (r'[0-9]+', Number.Integer),
+            (r'"(\\\\|\\"|[^"])*"', String.Double),
+            (r"'(\\\\|\\'|[^'])*'", String.Single),
+            (r'[~\^\*!%&<>\|+=:;,/?\\{}\[\]();.-]+', Operator),
+        ],
+        'funcparams': [
+            (r'(' + identifier + r')(\s*)(:)(\s*)(' + identifier + r'|\*)(\s*)(,?)',
+             bygroups(Name, Text, Operator, Text, Keyword.Type, Text, Operator)),
+            (r'\)', Operator, 'type')
+        ],
+        'type': [
+            (r'(\s*)(:)(' + identifier + r'|\*)',
+             bygroups(Text, Operator, Keyword.Type), '#pop:2')
+        ]
+    }
+
+
 class CssLexer(RegexLexer):
     """
     For CSS (Cascading Style Sheets).
@@ -78,8 +229,6 @@ class CssLexer(RegexLexer):
 
     tokens = {
         'root': [
-            (r'(@media)(\s+)(\w+)(\s*)({)', bygroups(Keyword, Text, String,
-             Text, Punctuation), 'media'),
             include('basics'),
         ],
         'basics': [
@@ -89,14 +238,20 @@ class CssLexer(RegexLexer):
             (r'\:[a-zA-Z0-9_-]+', Name.Decorator),
             (r'\.[a-zA-Z0-9_-]+', Name.Class),
             (r'\#[a-zA-Z0-9_-]+', Name.Function),
+            (r'@[a-zA-Z0-9_-]+', Keyword, 'atrule'),
             (r'[a-zA-Z0-9_-]+', Name.Tag),
             (r'[~\^\*!%&\[\]\(\)<>\|+=@:;,./?-]', Operator),
             (r'"(\\\\|\\"|[^"])*"', String.Double),
             (r"'(\\\\|\\'|[^'])*'", String.Single)
         ],
-        'media': [
+        'atrule': [
+            (r'{', Punctuation, 'atcontent'),
+            (r';', Punctuation, '#pop'),
             include('basics'),
-            (r'}', Punctuation, '#pop')
+        ],
+        'atcontent': [
+            include('basics'),
+            (r'}', Punctuation, '#pop:2'),
         ],
         'content': [
             (r'\s+', Text),
@@ -207,14 +362,14 @@ class HtmlLexer(RegexLexer):
 
     name = 'HTML'
     aliases = ['html']
-    filenames = ['*.html', '*.htm', '*.xhtml']
+    filenames = ['*.html', '*.htm', '*.xhtml', '*.xslt']
     mimetypes = ['text/html', 'application/xhtml+xml']
 
     flags = re.IGNORECASE | re.DOTALL
     tokens = {
         'root': [
             ('[^<&]+', Text),
-            ('&.*?;', Name.Entity),
+            (r'&\S*?;', Name.Entity),
             (r'\<\!\[CDATA\[.*?\]\]\>', Comment.Preproc),
             ('<!--', Comment, 'comment'),
             (r'<\?.*?\?>', Comment.Preproc),
@@ -264,7 +419,8 @@ class PhpLexer(RegexLexer):
 
     `startinline`
         If given and ``True`` the lexer starts highlighting with
-        php code. (i.e.: no starting ``<?php`` required)
+        php code (i.e.: no starting ``<?php`` required).  The default
+        is ``False``.
     `funcnamehighlighting`
         If given and ``True``, highlight builtin function names
         (default: ``True``).
@@ -305,6 +461,7 @@ class PhpLexer(RegexLexer):
             (r'\s+', Text),
             (r'#.*?\n', Comment),
             (r'//.*?\n', Comment),
+            (r'/\*\*.*?\*/', String.Doc),
             (r'/\*.*?\*/', Comment),
             (r'(->|::)(\s*)([a-zA-Z_][a-zA-Z0-9_]*)',
              bygroups(Operator, Text, Name.Attribute)),
@@ -326,7 +483,8 @@ class PhpLexer(RegexLexer):
              r'implements|public|private|protected|abstract|clone|try|'
              r'catch|throw|this)\b', Keyword),
             ('(true|false|null)\b', Keyword.Constant),
-            (r'\$[a-zA-Z_][a-zA-Z0-9_]*', Name.Variable),
+            (r'\$\{\$+[a-zA-Z_][a-zA-Z0-9_]*\}', Name.Variable),
+            (r'\$+[a-zA-Z_][a-zA-Z0-9_]*', Name.Variable),
             ('[a-zA-Z_][a-zA-Z0-9_]*', Name.Other),
             (r"[0-9](\.[0-9]*)?(eE[+-][0-9])?[flFLdD]?|"
              r"0[xX][0-9a-fA-F]+[Ll]?", Number),
@@ -355,7 +513,7 @@ class PhpLexer(RegexLexer):
             (r'(\$\{)(\S+)(\})',
              bygroups(String.Interpol, Name.Variable, String.Interpol)),
             (r'[${\\]+', String.Double)
-        ]
+        ],
     }
 
     def __init__(self, **options):
@@ -386,15 +544,16 @@ class PhpLexer(RegexLexer):
             RegexLexer.get_tokens_unprocessed(self, text, stack):
             if token is Name.Other:
                 if value in self._functions:
-                    yield index, Name.Function, value
+                    yield index, Name.Builtin, value
                     continue
             yield index, token, value
 
     def analyse_text(text):
         rv = 0.0
-        for tag in '<?php', '?>':
-            if tag in text:
-                rv += 0.2
+        if re.search(r'<\?(?!xml)', text):
+            rv += 0.3
+        if '?>' in text:
+            rv += 0.1
         return rv
 
 
@@ -407,7 +566,7 @@ class XmlLexer(RegexLexer):
 
     name = 'XML'
     aliases = ['xml']
-    filenames = ['*.xml', '*.xsl', '*.rss']
+    filenames = ['*.xml', '*.xsl', '*.rss', '*.xslt', '*.xsd', '*.wsdl']
     mimetypes = ['text/xml', 'application/xml', 'image/svg+xml',
                  'application/rss+xml', 'application/atom+xml',
                  'application/xsl+xml', 'application/xslt+xml']
@@ -415,7 +574,7 @@ class XmlLexer(RegexLexer):
     tokens = {
         'root': [
             ('[^<&]+', Text),
-            ('&.*?;', Name.Entity),
+            (r'&\S*?;', Name.Entity),
             (r'\<\!\[CDATA\[.*?\]\]\>', Comment.Preproc),
             ('<!--', Comment, 'comment'),
             (r'<\?.*?\?>', Comment.Preproc),
@@ -444,3 +603,39 @@ class XmlLexer(RegexLexer):
     def analyse_text(text):
         if looks_like_xml(text):
             return 0.5
+
+
+class XsltLexer(XmlLexer):
+    '''
+    A lexer for XSLT.
+
+    *New in Pygments 0.10.*
+    '''
+
+    name = 'XSLT'
+    aliases = ['xslt']
+    filenames = ['*.xsl', '*.xslt']
+
+    EXTRA_KEYWORDS = set([
+        'apply-imports', 'apply-templates', 'attribute',
+        'attribute-set', 'call-template', 'choose', 'comment',
+        'copy', 'copy-of', 'decimal-format', 'element', 'fallback',
+        'for-each', 'if', 'import', 'include', 'key', 'message',
+        'namespace-alias', 'number', 'otherwise', 'output', 'param',
+        'preserve-space', 'processing-instruction', 'sort',
+        'strip-space', 'stylesheet', 'template', 'text', 'transform',
+        'value-of', 'variable', 'when', 'with-param'
+    ])
+
+    def get_tokens_unprocessed(self, text):
+        for index, token, value in XmlLexer.get_tokens_unprocessed(self, text):
+            m = re.match('</?xsl:([^>]*)/?>?', value)
+
+            if token is Name.Tag and m and m.group(1) in self.EXTRA_KEYWORDS:
+                yield index, Keyword, value
+            else:
+                yield index, token, value
+
+    def analyse_text(text):
+        if looks_like_xml(text) and '<xsl' in text:
+            return 0.8
